'''
Flask-Jy-Share
~~~~~~~~~~~~~~~
Create social share component in Jinja2 template based on share.js.
:copyright: (c) 2020 by jiangyang.
:license: MIT, see LICENSE for more details.
'''
import re

from flask import (
    current_app,
    url_for,
    Markup,
    Blueprint,
    request,
)


class Share(object):
    def __init__(self, app=None):
        if app is not None:
            self.init_app(app)

    def init_app(self, app):
        '''
        初始化
        '''
        if not hasattr(app, 'extensions'):
            app.extensions = {}
        # 注册扩展到 app.extensions 字典
        app.extensions['share'] = self
        
        # 把扩展类添加到模板上下文
        app.jinja_env.globals['share'] = self
        
        # 添加扩展配置
        app.config.setdefault('SHARE_SITES', 'weibo, wechat, douban, facebook, twitter, google, linkedin, qq, qzone')
        app.config.setdefault('SHARE_MOBILE_SITES', 'weibo, douban, qq, qzone')
        app.config.setdefault('SHARE_HIDE_ON_MOBILE', False)
        
        # 支持用户设置是否使用内置资源
        app.config.setdefault('SHARE_SERVE_LOCAL', False)
        # 创建蓝本
        blueprint = Blueprint('share', __name__, static_folder='static',
                static_url_path='/share' + app.static_url_path)
        app.register_blueprint(blueprint)

    @staticmethod
    def load(css_url=None, js_url=None):
        '''
        加载 share.js 资源

        :param css_url: 如果设置了, 就会使用这个 css_url
        :param js_url: 如果设置了, 就会使用这个 js_url
        :param serve_local: 如果设置为 True, 就使用本地资源
        '''
        if current_app.config['SHARE_SERVE_LOCAL']:
            css_url = url_for('share.static', filename='css/share.min.css')
            js_url = url_for('share.static', filename='js/share.min.js')
        if css_url is None:
            css_url = 'https://cdn.bootcss.com/social-share.js/1.0.16/css/share.min.css'
        if js_url is None:
            js_url = 'https://cdn.bootcss.com/social-share.js/1.0.16/js/social-share.min.js'
        return Markup('''<link rel="stylesheet" href="%s" type="text/css">\n
            <script src="%s"></script>''' % (css_url, js_url))

    @staticmethod
    def create(title='', sites=None, mobile_sites=None, align='left', addition_class=''):
        '''
        创建一个 share 组件.

        :param title: 标题.
        :param sites: 以 , 分隔的由站点组成的字符串, 支持的站点 name 有: weibo, wechat, douban, facebook, twitter, google, linkedin, qq, qzone.例如: `'weibo, wechat, qq'`.
        :param mobile_sites: 手机上显示的网站.
        :param align: 组件的对齐方式, 默认为 left.
        :param addition_class: the style class added to the share component.
        '''
        # 在移动设备上隐藏
        if current_app.config['SHARE_HIDE_ON_MOBILE']:
            platform = request.user_agent.platform
            if platform is not None:
                mobile_pattern = re.compile('android|fennec|iemobile|iphone|opera(?:mini|mobi)')
            
                m = re.match(mobile_pattern, platform)
                if m is not None:
                    return ''

        if sites is None:
            sites = current_app.config['SHARE_SITES']
        if mobile_sites is None:
            mobile_sites = current_app.config['SHARE_MOBILE_SITES']
        return Markup('''<div class="social-share %s" data-sites="%s" data-mobile-sites="%s" align="%s">%s</div>
            ''' % (addition_class, sites, mobile_sites, align, title))



'''
test_flask_share
~~~~~~~~~~~~~~~
Create social share component in Jinja2 template based on share.js.
:copyright: (c) 2020 by jiangyang.
:license: MIT, see LICENSE for more details.
'''

import unittest

from flask import (
    Flask,
    render_template_string,
    current_app,
)

from flask_share import Share


class ShareTestCase(unittest.TestCase):
    def setUp(self):
        self.mobile_agent = {
            'HTTP_USER_AGENT': 'Mozilla/5.0 (iPhone; CPU iPhone OS 9_1 like Mac OS X) \
                    AppleWebKit/601.1.46 (KHTML, like Gecko) Version/9.0 Mobile/13B143 Safari/601.1',
            }

        app = Flask(__name__)
        app.testing = True
        self.share = Share(app)

        @app.route('/')
        def index():
            return render_template_string('{{ share.load() }}\n{{ share.create() }}')

        self.context = app.app_context()
        self.context.push()
        self.client = app.test_client()

    def tearDown(self):
        self.context.pop()

    def test_config(self):
        self.assertIn('SHARE_SITES', current_app.config)
        self.assertIn('SHARE_MOBILE_SITES', current_app.config)
        self.assertIn('SHARE_HIDE_ON_MOBILE', current_app.config)
        self.assertIn('SHARE_SERVE_LOCAL', current_app.config)
        self.assertEqual(current_app.config['SHARE_HIDE_ON_MOBILE'], False)

    def test_load(self):
        rv = self.share.load()
        self.assertIn('https://cdn.bootcss.com', rv)
        self.assertIn('social-share.min.js', rv)

        response = self.client.get('/')
        data = response.get_data(as_text=True)
        self.assertIn('social-share.min.js', data)

    def test_create(self):
        rv = self.share.create()
        self.assertIn('<div class="social-share', rv)

        response = self.client.get('/')
        data = response.get_data(as_text=True)
        self.assertIn('<div class="social-share', data)

    def test_custom_sites(self):
        current_app.config['SHARE_SITES'] = 'twitter, facebook'
        response = self.client.get('/')
        data = response.get_data(as_text=True)
        self.assertIn('data-sites="twitter, facebook"', data)

    def test_custom_mobile_sites(self):
        current_app.config['SHARE_MOBILE_SITES'] = 'twitter'
        response = self.client.get('/', environ_base=self.mobile_agent)
        data = response.get_data(as_text=True)
        self.assertIn('data-mobile-sites="twitter"', data)

    def test_hide_on_mobile_config(self):
        current_app.config['SHARE_HIDE_ON_MOBILE'] = True
        response = self.client.get('/')
        data = response.get_data(as_text=True)
        self.assertIn('social-share.min.js', data)
        self.assertIn('<div class="social-share', data)

    def test_create_on_mobile(self):
        current_app.config['SHARE_HIDE_ON_MOBILE'] = True
        response = self.client.get('/', environ_base=self.mobile_agent)
        data = response.get_data(as_text=True)
        self.assertIn('social-share.min.js', data)
        self.assertNotIn('<div class="social-share', data)

    def test_local_resources(self):
        current_app.config['SHARE_SERVE_LOCAL'] = True

        response = self.client.get('/share/static/css/share.min.css')
        self.assertNotEqual(response.status_code, 404)



'''
Flask-Jy-Share
~~~~~~~~~~~~~~~
Create social share component in Jinja2 template based on share.js.
:copyright: (c) 2020 by jiangyang.
:license: MIT, see LICENSE for more details.
'''
from os import path
from codecs import open
from setuptools import setup

basedir = path.abspath(path.dirname(__file__))


# 从 README 中读取描述
with open(path.join(basedir, 'README.md'), encoding='utf-8') as f:
    long_description = f.read()

setup(
    name='Flask-Jy-Share',
    version='0.1.0',
    url='https://github.com/jiangyanglinlan/flask-share',
    license='MIT',
    author='jiangyang',
    author_email='jiangyanglinlan@qq.com',
    description='Create social share component in Jinja2 template based on share.js.',
    long_description=long_description,
    long_description_content_type='text/markdown',  # 长描述内容模型
    platforms='any',
    packages=['flask_share'],  # 包含的包列表
    zip_safe=False,
    test_suite='test_flask_share',
    include_package_data=True,
    install_requires=[
        'Flask',
    ],
    keywords='flask extension development',
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
